/*
 *  Copyright 2008 CNRS
 *  Author: Florent Lamiraux
 */

#include "KineoModel/kppLicense.h"
#include "hppWalkPlanner/hppWalkPlanner.h"
#include "testDirectPath.h"
#include "kwsIO/kwsioConfig.h"
#include "KineoWorks2/kwsDPLinear.h"
#include "KineoWorks2/kwsDof.h"

int main (int argc, char** argv)
{
  if(!CkppLicense::initialize()) {
    std::cerr << "Error: license for KPP SDK was not found" << std::endl;
    return -1;
  }
  else {
    std::cout << "license for KPP SDK was found" << std::endl;
  }

  CtestDirectPath testDirectPath;
  testDirectPath.init();

  CkppDeviceComponentShPtr device = testDirectPath.device();
  CkwsSteeringMethodShPtr steeringMethod = 
    testDirectPath.steeringMethod();

  for (unsigned int iDof=0; iDof < device->countDofs(); iDof++) {
    CkwsDofShPtr dof = device->dof(iDof);
    std::cout << "dof " << iDof << ", weight = " << dof->weight() 
	      << ", userWeightScaler = " << dof->userWeightScaler() << std::endl;
  }

  // Test a few direct paths
  CkwsConfig config1(device);
  CkwsConfig config2(device);
  CkwsDirectPathShPtr directPath;

  config1.dofValue(0, 0);
  config1.dofValue(1, 0);
  config1.dofValue(2, 0);
  config1.dofValue(6, 0);

  config2.dofValue(0, 1);
  config2.dofValue(1, 0);
  config2.dofValue(2, 0);
  config2.dofValue(6, 0);

  directPath = steeringMethod->makeDirectPath(config1, config2);

  std::cout << "config1 = (" << config1 << ")" << std::endl;
  std::cout << "config2 = (" << config2 << ")" << std::endl;
  if (directPath) {
    std::cout << "direct path length = " << directPath->length() << "." 
	      << std::endl << std::endl;  
  }

  config1.dofValue(0, 0);
  config1.dofValue(1, 0);
  config1.dofValue(2, 0);
  config1.dofValue(6, 0);

  config2.dofValue(0, 0);
  config2.dofValue(1, 1);
  config2.dofValue(2, 0);
  config2.dofValue(6, 0);

  directPath = steeringMethod->makeDirectPath(config1, config2);

  std::cout << "config1 = (" << config1 << ")" << std::endl;
  std::cout << "config2 = (" << config2 << ")" << std::endl;
  if (directPath) {
    std::cout << "direct path length = " << directPath->length() << "." 
	      << std::endl << std::endl;  
  }

  config1.dofValue(0, 0);
  config1.dofValue(1, 0);
  config1.dofValue(2, 0);
  config1.dofValue(6, 0);

  config2.dofValue(0, 0);
  config2.dofValue(1, 0);
  config2.dofValue(2, 1);
  config2.dofValue(6, 0);

  directPath = steeringMethod->makeDirectPath(config1, config2);

  std::cout << "config1 = (" << config1 << ")" << std::endl;
  std::cout << "config2 = (" << config2 << ")" << std::endl;
  if (directPath) {
    std::cout << "direct path length = " << directPath->length() << "." 
	      << std::endl << std::endl;  
  }

  config1.dofValue(0, 0);
  config1.dofValue(1, 0);
  config1.dofValue(2, 0);
  config1.dofValue(6, 0);

  config2.dofValue(0, 0);
  config2.dofValue(1, 0);
  config2.dofValue(2, 0);
  config2.dofValue(6, 1);

  directPath = steeringMethod->makeDirectPath(config1, config2);

  std::cout << "config1 = (" << config1 << ")" << std::endl;
  std::cout << "config2 = (" << config2 << ")" << std::endl;
  if (directPath) {
    std::cout << "direct path length = " << directPath->length() << "." 
	      << std::endl << std::endl;  
  }

  config1.dofValue(0, 0);
  config1.dofValue(1, 0);
  config1.dofValue(2, 0);
  config1.dofValue(6, 0);

  config2.dofValue(0, 1);
  config2.dofValue(1, 1);
  config2.dofValue(2, 1);
  config2.dofValue(6, 1);

  directPath = steeringMethod->makeDirectPath(config1, config2);

  std::cout << "config1 = (" << config1 << ")" << std::endl;
  std::cout << "config2 = (" << config2 << ")" << std::endl;
  if (directPath) {
    std::cout << "direct path length = " << directPath->length() << "." 
	      << std::endl << std::endl;  
  }

  return 0 ; 
}

